@@ -1,12 +1,14 @@
 import {AutocompleteField, DropDownMenu, FormField, FormSelect, HelpIcon, PopupApi} from 'argo-ui';
 import * as React from 'react';
 import {FormApi, Text} from 'react-form';
-import {Cluster, clusterTitle, DataLoader, EditablePanel, EditablePanelItem, Expandable, MapInputField, Repo, Revision, RevisionHelpIcon} from '../../../shared/components';
+import {Cluster, DataLoader, EditablePanel, EditablePanelItem, Expandable, MapInputField, NumberField, Repo, Revision, RevisionHelpIcon} from '../../../shared/components';
+import {BadgePanel, Spinner} from '../../../shared/components';
 import {Consumer} from '../../../shared/context';
 import * as models from '../../../shared/models';
 import {services} from '../../../shared/services';
 
 import {ApplicationSyncOptionsField} from '../application-sync-options';
+import {RevisionFormField} from '../revision-form-field/revision-form-field';
 import {ComparisonStatusIcon, HealthStatusIcon, syncStatusMessage} from '../utils';
 
 require('./application-summary.scss');
@@ -28,13 +30,15 @@ function swap(array: any[], a: number, b: number) {
 export const ApplicationSummary = (props: {app: models.Application; updateApp: (app: models.Application) => Promise<any>}) => {
     const app = JSON.parse(JSON.stringify(props.app)) as models.Application;
     const isHelm = app.spec.source.hasOwnProperty('chart');
-
+    const initialState = app.spec.destination.server === undefined ? 'NAME' : 'URL';
+    const [destFormat, setDestFormat] = React.useState(initialState);
+    const [changeSync, setChangeSync] = React.useState(false);
     const attributes = [
         {
             title: 'PROJECT',
             view: <a href={'/settings/projects/' + app.spec.project}>{app.spec.project}</a>,
             edit: (formApi: FormApi) => (
-                <DataLoader load={() => services.projects.list().then(projs => projs.map(item => item.metadata.name))}>
+                <DataLoader load={() => services.projects.list('items.metadata.name').then(projs => projs.map(item => item.metadata.name))}>
                     {projects => <FormField formApi={formApi} field='spec.project' component={FormSelect} componentProps={{options: projects}} />}
                 </DataLoader>
             )
@@ -59,18 +63,62 @@ export const ApplicationSummary = (props: {app: models.Application; updateApp: (
         },
         {
             title: 'CLUSTER',
-            view: <Cluster server={app.spec.destination.server} showUrl={true} />,
+            view: <Cluster server={app.spec.destination.server} name={app.spec.destination.name} showUrl={true} />,
             edit: (formApi: FormApi) => (
-                <DataLoader
-                    load={() =>
-                        services.clusters.list().then(clusters =>
-                            clusters.map(cluster => ({
-                                title: clusterTitle(cluster),
-                                value: cluster.server
-                            }))
-                        )
-                    }>
-                    {clusters => <FormField formApi={formApi} field='spec.destination.server' componentProps={{options: clusters}} component={FormSelect} />}
+                <DataLoader load={() => services.clusters.list().then(clusters => clusters.sort())}>
+                    {clusters => {
+                        return (
+                            <div className='row'>
+                                {(destFormat.toUpperCase() === 'URL' && (
+                                    <div className='columns small-10'>
+                                        <FormField
+                                            formApi={formApi}
+                                            field='spec.destination.server'
+                                            componentProps={{items: clusters.map(cluster => cluster.server)}}
+                                            component={AutocompleteField}
+                                        />
+                                    </div>
+                                )) || (
+                                    <div className='columns small-10'>
+                                        <FormField
+                                            formApi={formApi}
+                                            field='spec.destination.name'
+                                            componentProps={{items: clusters.map(cluster => cluster.name)}}
+                                            component={AutocompleteField}
+                                        />
+                                    </div>
+                                )}
+                                <div className='columns small-2'>
+                                    <div>
+                                        <DropDownMenu
+                                            anchor={() => (
+                                                <p>
+                                                    {destFormat.toUpperCase()} <i className='fa fa-caret-down' />
+                                                </p>
+                                            )}
+                                            items={['URL', 'NAME'].map((type: 'URL' | 'NAME') => ({
+                                                title: type,
+                                                action: () => {
+                                                    if (destFormat !== type) {
+                                                        const updatedApp = formApi.getFormState().values as models.Application;
+                                                        if (type === 'URL') {
+                                                            updatedApp.spec.destination.server = '';
+                                                            delete updatedApp.spec.destination.name;
+                                                        } else {
+                                                            updatedApp.spec.destination.name = '';
+                                                            delete updatedApp.spec.destination.server;
+                                                        }
+                                                        formApi.setAllValues(updatedApp);
+                                                        setDestFormat(type);
+                                                    }
+                                                }
+                                            }))}
+                                        />
+                                    </div>
+                                </div>
+                            </div>
+                        );
+                    }}
                 </DataLoader>
             )
         },
@@ -126,7 +174,7 @@ export const ApplicationSummary = (props: {app: models.Application; updateApp: (
                                                           items: versions
                                                       }}
                                                   />
-                                                  <RevisionHelpIcon type='helm' />
+                                                  <RevisionHelpIcon type='helm' top='0' />
                                               </div>
                                           )}
                                       </DataLoader>
@@ -140,12 +188,7 @@ export const ApplicationSummary = (props: {app: models.Application; updateApp: (
                   {
                       title: 'TARGET REVISION',
                       view: <Revision repoUrl={app.spec.source.repoURL} revision={app.spec.source.targetRevision || 'HEAD'} />,
-                      edit: (formApi: FormApi) => (
-                          <React.Fragment>
-                              <FormField formApi={formApi} field='spec.source.targetRevision' component={Text} componentProps={{placeholder: 'HEAD'}} />
-                              <RevisionHelpIcon type='git' />{' '}
-                          </React.Fragment>
-                      )
+                      edit: (formApi: FormApi) => <RevisionFormField helpIconTop={'0'} hideLabel={true} formApi={formApi} repoURL={app.spec.source.repoURL} />
                   },
                   {
                       title: 'PATH',
@@ -158,23 +201,28 @@ export const ApplicationSummary = (props: {app: models.Application; updateApp: (
             title: 'REVISION HISTORY LIMIT',
             view: app.spec.revisionHistoryLimit,
             edit: (formApi: FormApi) => (
-                <React.Fragment>
-                    <FormField formApi={formApi} field='spec.revisionHistoryLimit' component={Text} />
-                    <HelpIcon
-                        title='This limits this number of items kept in the apps revision history.
-This should only be changed in exceptional circumstances.
-Setting to zero will store no history. This will reduce storage used.
-Increasing will increase the space used to store the history, so we do not recommend increasing it.
-Default is 10.
-'
-                    />
-                </React.Fragment>
+                <div style={{position: 'relative'}}>
+                    <FormField formApi={formApi} field='spec.revisionHistoryLimit' componentProps={{style: {paddingRight: '1em'}, placeholder: '10'}} component={NumberField} />
+                    <div style={{position: 'absolute', right: '0', top: '0'}}>
+                        <HelpIcon
+                            title='This limits this number of items kept in the apps revision history.
+    This should only be changed in exceptional circumstances.
+    Setting to zero will store no history. This will reduce storage used.
+    Increasing will increase the space used to store the history, so we do not recommend increasing it.
+    Default is 10.'
+                        />
+                    </div>
+                </div>
             )
         },
         {
             title: 'SYNC OPTIONS',
             view: ((app.spec.syncPolicy || {}).syncOptions || []).join(', '),
-            edit: (formApi: FormApi) => <FormField formApi={formApi} field='spec.syncPolicy.syncOptions' component={ApplicationSyncOptionsField} />
+            edit: (formApi: FormApi) => (
+                <div>
+                    <FormField formApi={formApi} field='spec.syncPolicy.syncOptions' component={ApplicationSyncOptionsField} />
+                </div>
+            )
         },
         {
             title: 'STATUS',
@@ -200,11 +248,13 @@ Default is 10.
             title: 'URLs',
             view: (
                 <React.Fragment>
-                    {urls.map(item => (
-                        <a key={item} href={item} target='__blank'>
-                            {item} &nbsp;
-                        </a>
-                    ))}
+                    {urls
+                        .map(item => item.split('|'))
+                        .map((parts, i) => (
+                            <a key={i} href={parts.length > 1 ? parts[1] : parts[0]} target='__blank'>
+                                {parts[0]} &nbsp;
+                            </a>
+                        ))}
                 </React.Fragment>
             )
         });
@@ -228,21 +278,31 @@ Default is 10.
     async function setAutoSync(ctx: {popup: PopupApi}, confirmationTitle: string, confirmationText: string, prune: boolean, selfHeal: boolean) {
         const confirmed = await ctx.popup.confirm(confirmationTitle, confirmationText);
         if (confirmed) {
-            const updatedApp = JSON.parse(JSON.stringify(props.app)) as models.Application;
-            if (!updatedApp.spec.syncPolicy) {
-                updatedApp.spec.syncPolicy = {};
+            try {
+                setChangeSync(true);
+                const updatedApp = JSON.parse(JSON.stringify(props.app)) as models.Application;
+                if (!updatedApp.spec.syncPolicy) {
+                    updatedApp.spec.syncPolicy = {};
+                }
+                updatedApp.spec.syncPolicy.automated = {prune, selfHeal};
+                await props.updateApp(updatedApp);
+            } finally {
+                setChangeSync(false);
             }
-            updatedApp.spec.syncPolicy.automated = {prune, selfHeal};
-            props.updateApp(updatedApp);
         }
     }
 
     async function unsetAutoSync(ctx: {popup: PopupApi}) {
         const confirmed = await ctx.popup.confirm('Disable Auto-Sync?', 'Are you sure you want to disable automated application synchronization');
         if (confirmed) {
-            const updatedApp = JSON.parse(JSON.stringify(props.app)) as models.Application;
-            updatedApp.spec.syncPolicy.automated = null;
-            props.updateApp(updatedApp);
+            try {
+                setChangeSync(true);
+                const updatedApp = JSON.parse(JSON.stringify(props.app)) as models.Application;
+                updatedApp.spec.syncPolicy.automated = null;
+                await props.updateApp(updatedApp);
+            } finally {
+                setChangeSync(false);
+            }
         }
     }
 
@@ -318,9 +378,6 @@ Default is 10.
             view: null as any,
             edit: null
         });
-    const [badgeType, setBadgeType] = React.useState('URL');
-    const badgeURL = `${location.protocol}//${location.host}/api/badge?name=${props.app.metadata.name}&revision=true`;
-    const appURL = `${location.protocol}//${location.host}/applications/${props.app.metadata.name}`;
 
     return (
         <div className='application-summary'>
@@ -328,8 +385,8 @@ Default is 10.
                 save={props.updateApp}
                 validate={input => ({
                     'spec.project': !input.spec.project && 'Project name is required',
-                    'spec.destination.server': !input.spec.destination.server && 'Cluster is required',
-                    'spec.destination.namespace': !input.spec.destination.namespace && 'Namespace is required'
+                    'spec.destination.server': !input.spec.destination.server && input.spec.destination.hasOwnProperty('server') && 'Cluster server is required',
+                    'spec.destination.name': !input.spec.destination.name && input.spec.destination.hasOwnProperty('name') && 'Cluster name is required'
                 })}
                 values={app}
                 title={app.metadata.name.toLocaleUpperCase()}
@@ -345,6 +402,7 @@ Default is 10.
                                 <div className='columns small-9'>
                                     {(app.spec.syncPolicy && app.spec.syncPolicy.automated && (
                                         <button className='argo-button argo-button--base' onClick={() => unsetAutoSync(ctx)}>
+                                            <Spinner show={changeSync} style={{marginRight: '5px'}} />
                                             Disable Auto-Sync
                                         </button>
                                     )) || (
@@ -353,6 +411,7 @@ Default is 10.
                                             onClick={() =>
                                                 setAutoSync(ctx, 'Enable Auto-Sync?', 'Are you sure you want to enable automated application synchronization?', false, false)
                                             }>
+                                            <Spinner show={changeSync} style={{marginRight: '5px'}} />
                                             Enable Auto-Sync
                                         </button>
                                     )}
@@ -435,48 +494,7 @@ Default is 10.
                     </div>
                 )}
             </Consumer>
-            <DataLoader load={() => services.authService.settings()}>
-                {settings =>
-                    (settings.statusBadgeEnabled && (
-                        <div className='white-box'>
-                            <div className='white-box__details'>
-                                <p>
-                                    Status Badge <img src={badgeURL} />{' '}
-                                </p>
-                                <div className='white-box__details-row'>
-                                    <DropDownMenu
-                                        anchor={() => (
-                                            <p>
-                                                {badgeType} <i className='fa fa-caret-down' />
-                                            </p>
-                                        )}
-                                        items={['URL', 'Markdown', 'Textile', 'Rdoc', 'AsciiDoc'].map(type => ({title: type, action: () => setBadgeType(type)}))}
-                                    />
-                                    <textarea
-                                        onClick={e => (e.target as HTMLInputElement).select()}
-                                        className='application-summary__badge'
-                                        readOnly={true}
-                                        value={
-                                            badgeType === 'URL'
-                                                ? badgeURL
-                                                : badgeType === 'Markdown'
-                                                ? `[![App Status](${badgeURL})](${appURL})`
-                                                : badgeType === 'Textile'
-                                                ? `!${badgeURL}!:${appURL}`
-                                                : badgeType === 'Rdoc'
-                                                ? `{<img src="${badgeURL}" alt="App Status" />}[${appURL}]`
-                                                : badgeType === 'AsciiDoc'
-                                                ? `image:${badgeURL}["App Status", link="${appURL}"]`
-                                                : ''
-                                        }
-                                    />
-                                </div>
-                            </div>
-                        </div>
-                    )) ||
-                    null
-                }
-            </DataLoader>
+            <BadgePanel app={props.app.metadata.name} />
             <EditablePanel save={props.updateApp} values={app} title='Info' items={infoItems} onModeSwitch={() => setAdjustedCount(0)} />
         </div>
     );
